import * as React from 'react';
import { DelayedTask } from '../../util';
import { ApplicationTranslatorT } from '../../translation';
import { IApplicationSettings } from '../../settings';
import { IDateConverter, INumberConverter } from '../../converter';
import { IUniversalComponentPlugin, IUniversalComponent, IUniversalComponentPluginClassEntity } from '../../entities-definitions.interface';
import { IUniversalComponentProps } from '../../props-definitions.interface';
import { IUIFactory } from '../factory/factory.interface';
import { IApplicationDomAccessor } from '../dom-accessor/dom-accessor.interface';
export declare class UniversalComponent<TComponent extends IUniversalComponent<TProps, TState>, TProps extends IUniversalComponentProps = IUniversalComponentProps, TState = {}> extends React.Component<TProps, TState> implements IUniversalComponent<TProps, TState> {
    protected plugins: IUniversalComponentPlugin[];
    protected scrollTask: DelayedTask;
    protected selfRef: React.RefObject<Element>;
    /**
     * @stable [23.04.2018]
     * @param {TProps} props
     */
    constructor(props: TProps);
    /**
     * @stable [23.04.2018]
     */
    componentDidMount(): void;
    /**
     * @stable [18.12.2018]
     * @param {Readonly<TProps extends IUniversalComponentProps>} prevProps
     * @param {Readonly<TState>} prevState
     */
    getSnapshotBeforeUpdate(prevProps: Readonly<TProps>, prevState: Readonly<TState>): void;
    /**
     * @stable [23.04.2018]
     */
    componentWillUnmount(): void;
    /**
     * @stable [23.04.2018]
     * @param {Readonly<TProps>} prevProps
     * @param {Readonly<TState>} prevState
     * @param {never} prevContext
     */
    componentDidUpdate(prevProps: Readonly<TProps>, prevState: Readonly<TState>, prevContext?: never): void;
    /**
     * @stable [01.12.2018]
     * @returns {Element}
     */
    getSelf(): Element;
    /**
     * @stable [01.12.2018]
     */
    protected onNativeScroll(): void;
    /**
     * @stable [01.12.2018]
     */
    protected doScroll(): void;
    /**
     * @stable [19.04.2018]
     * @returns {ApplicationTranslatorT}
     */
    protected readonly t: ApplicationTranslatorT;
    /**
     * @stable [19.04.2018]
     * @returns {IApplicationSettings}
     */
    protected readonly settings: IApplicationSettings;
    /**
     * @stable [23.04.2018]
     * @returns {Map<IUniversalComponentClassEntity, UniversalComponentPluginFactoryT>}
     */
    private readonly uiPlugins;
    /**
     * @stable [13.05.2018]
     * @returns {IDateConverter}
     */
    protected readonly dc: IDateConverter;
    /**
     * @stable [13.05.2018]
     * @returns {INumberConverter}
     */
    protected readonly nc: INumberConverter;
    /**
     * @stable [18.05.2018]
     * @returns {IUIFactory}
     */
    protected readonly uiFactory: IUIFactory;
    /**
     * @stable [01.12.2018]
     * @returns {IApplicationDomAccessor}
     */
    protected readonly domAccessor: IApplicationDomAccessor;
    /**
     * @stable [01.12.2018]
     * @returns {React.RefObject<TElement extends Element>}
     */
    protected getSelfRef<TElement extends Element>(): React.RefObject<TElement>;
    /**
     * @stable [23.04.2018]
     * @param {IUniversalComponentPluginClassEntity} pluginClassEntity
     * @returns {IUniversalComponentPlugin}
     */
    protected registerPlugin(pluginClassEntity: IUniversalComponentPluginClassEntity): IUniversalComponentPlugin;
}
