import { IEntity, AnyT } from '../../../definitions.interface';
import { IMultiItemEntity, MultiItemEntityT } from '../../../entities-definitions.interface';
import { VueField } from '../field/vue-index';
import { MultiFieldPlugin } from '../multifield/vue-index';
import { IVueBaseFileViewerProps, IVueViewerListenersEntity } from '../../viewer/vue-index';
import { IVueBaseFileFieldProps, IVueBaseFileFieldTemplateMethods } from './vue-filefield.interface';
export declare class VueBaseFileField extends VueField implements IVueBaseFileFieldTemplateMethods, IVueBaseFileFieldProps {
    readonly displayFileName: string;
    readonly displayFileFormat: string;
    readonly placeholderFactory: (index: number) => string;
    readonly maxFiles: number;
    readonly defaultDndMessage: string;
    readonly defaultDndMessageFactory: (index: number) => string;
    readonly viewer: string;
    protected readonly type: string;
    protected readonly viewerProps: IVueBaseFileViewerProps;
    protected readonly multiFieldPlugin: MultiFieldPlugin;
    protected readonly cachedFiles: Map<string, File>;
    /**
     * @stable [27.11.2018]
     */
    constructor();
    /**
     * @stable [25.11.2018]
     * @returns {string}
     */
    getFieldClassName(): string;
    /**
     * @stable [16.12.2018]
     * @param {File[]} files
     * @returns {string[]}
     */
    onFilesSelect(files: File[]): string[];
    /**
     * @stable [22.12.2018]
     * @param {number} index
     * @returns {string}
     */
    getDefaultDndMessage(index: number): string;
    /**
     * [<Placeholder: optional> + <Default DnD message>]
     *
     * @stable [22.12.2018]
     * @param {number} index
     * @returns {string}
     */
    getPlaceholder(index: number): string;
    /**
     * @stable [20.12.2018]
     * @returns {IEntity[]}
     */
    getFiles(): IEntity[];
    /**
     * @stable [22.12.2018]
     * @param {MultiItemEntityT} entityOrEntityId
     * @param {number} index
     * @returns {IVueBaseFileViewerProps}
     */
    getViewerBindings(entityOrEntityId: MultiItemEntityT, index: number): IVueBaseFileViewerProps;
    /**
     * @stable [16.12.2018]
     * @param {IMultiItemEntity} fileEntity
     * @param {number} index
     * @returns {IVueViewerListenersEntity<File>}
     */
    getViewerListeners(fileEntity: IMultiItemEntity, index: number): IVueViewerListenersEntity<File>;
    /**
     * @stable [21.12.2018]
     * @param {File} file
     * @param {number} index
     * @returns {string}
     */
    getViewerComponent(file: File, index: number): string;
    /**
     * @stable [06.01.2019]
     * @param {number} index
     * @returns {string}
     */
    getAttachmentContentClassName(index: number): string;
    /**
     * @stable [17.11.2018]
     * @returns {string}
     */
    protected getInputClassName(): string;
    /**
     * @stable [09.12.2018]
     * @returns {string}
     */
    protected getFieldAttachmentTemplate(): string;
    /**
     * @stable [16.12.2018]
     * @param {IMultiItemEntity} fileEntity
     * @param {File} file
     */
    protected onFileChange(fileEntity: IMultiItemEntity, file: File): void;
    /**
     * @stable [10.12.2018]
     * @returns {TMethods}
     */
    protected getTemplateMethods<TMethods extends IVueBaseFileFieldTemplateMethods>(): TMethods;
    protected getFileFormat(): AnyT;
    /**
     * @stable [19.01.2019]
     * @param {string} type
     * @returns {string}
     */
    protected toViewer(type: string): string;
    /**
     * @stable [10.12.2018]
     * @param {File} file
     * @returns {string}
     */
    private toFileUrl;
    /**
     * @stable [28.11.2018]
     * @param {string} fileUrl
     */
    private destroyFileUrl;
    /**
     * @stable [22.12.2018]
     * @param {File} file
     * @param {string} fileUrl
     */
    private addFile;
    /**
     * @stable [28.11.2018]
     * @param {IMultiItemEntity} fileEntity
     */
    private removeFile;
    private getFileName;
}
