"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redux_effects_promise_1 = require("redux-effects-promise");
var util_1 = require("../../util");
var definitions_interface_1 = require("../../definitions.interface");
var form_interface_1 = require("./form.interface");
var FormActionBuilder = /** @class */ (function () {
    function FormActionBuilder() {
    }
    /**
     * @stable [21.01.2019]
     * @param {string} section
     * @returns {string}
     */
    FormActionBuilder.buildDestroyActionType = function (section) {
        return util_1.toActionPrefix(section) + "." + form_interface_1.FORM_DESTROY_ACTION_TYPE;
    };
    /**
     * @stable [26.08.2018]
     * @param {string} section
     * @returns {string}
     */
    FormActionBuilder.buildClearActionType = function (section) {
        return util_1.toActionPrefix(section) + "." + form_interface_1.FORM_CLEAR_ACTION_TYPE;
    };
    /**
     * @stable [02.10.2018]
     * @param {string} section
     * @returns {string}
     */
    FormActionBuilder.buildChangeActionType = function (section) {
        return util_1.toActionPrefix(section) + "." + form_interface_1.FORM_CHANGE_ACTION_TYPE;
    };
    FormActionBuilder.buildResetActionType = function (section) {
        return section + "." + form_interface_1.FORM_RESET_ACTION_TYPE;
    };
    FormActionBuilder.buildValidActionType = function (section) {
        return section + "." + form_interface_1.FORM_VALID_ACTION_TYPE;
    };
    /**
     * @stable [14.08.2018]
     * @param {string} section
     * @returns {string}
     */
    FormActionBuilder.buildActiveValueActionType = function (section) {
        return util_1.toActionPrefix(section) + "." + form_interface_1.FORM_ACTIVE_VALUE_ACTION_TYPE;
    };
    /**
     * @stable [02.09.2018]
     * @param {string} section
     * @returns {string}
     */
    FormActionBuilder.buildDeactivatedValueActionType = function (section) {
        return util_1.toActionPrefix(section) + "." + form_interface_1.FORM_DEACTIVATED_VALUE_ACTION_TYPE;
    };
    FormActionBuilder.buildSubmitActionType = function (section) {
        return section + "." + form_interface_1.FORM_SUBMIT_ACTION_TYPE;
    };
    FormActionBuilder.buildSubmitDoneActionType = function (section) {
        return section + "." + form_interface_1.FORM_SUBMIT_DONE_ACTION_TYPE;
    };
    FormActionBuilder.buildSubmitFinishedActionType = function (section) {
        return section + "." + form_interface_1.FORM_SUBMIT_FINISHED_ACTION_TYPE;
    };
    FormActionBuilder.buildSubmitErrorActionType = function (section) {
        return section + "." + form_interface_1.FORM_SUBMIT_ERROR_ACTION_TYPE;
    };
    FormActionBuilder.buildSubmitDoneAction = function (section) {
        return redux_effects_promise_1.EffectsAction.create(this.buildSubmitDoneActionType(section), util_1.applySection(section));
    };
    FormActionBuilder.buildSubmitFinishedAction = function (section) {
        return redux_effects_promise_1.EffectsAction.create(this.buildSubmitFinishedActionType(section), util_1.applySection(section));
    };
    /**
     * @stable [21.01.2019]
     * @param {string} section
     * @returns {IEffectsAction}
     */
    FormActionBuilder.buildDestroyAction = function (section) {
        return redux_effects_promise_1.EffectsAction.create(this.buildDestroyActionType(section), util_1.applySection(section));
    };
    FormActionBuilder.buildChangeAction = function (section, data) {
        return redux_effects_promise_1.EffectsAction.create(this.buildChangeActionType(section), util_1.applySection(section, data));
    };
    /**
     * @stable [26.08.2018]
     * @param {string} section
     * @param {string} fieldName
     * @returns {IEffectsAction}
     */
    FormActionBuilder.buildClearSimpleAction = function (section, fieldName) {
        var _a;
        return {
            type: this.buildClearActionType(section),
            data: util_1.applySection(section, this.buildChangesPayload((_a = {}, _a[fieldName] = definitions_interface_1.UNDEF, _a))),
        };
    };
    /**
     * @stable - 10.04.2018
     * @param {string} section
     * @param {IKeyValue} changes
     * @returns {IEffectsAction}
     */
    FormActionBuilder.buildChangesAction = function (section, changes) {
        return redux_effects_promise_1.EffectsAction.create(this.buildChangeActionType(section), util_1.applySection(section, this.buildChangesPayload(changes)));
    };
    /**
     * @stable - 11.04.2018
     * @param {string} section
     * @param {IKeyValue} changes
     * @returns {AnyAction}
     */
    FormActionBuilder.buildChangesSimpleAction = function (section, changes) {
        return {
            type: this.buildChangeActionType(section),
            data: util_1.applySection(section, this.buildChangesPayload(changes)),
        };
    };
    /**
     * @stable - 25.04.2018
     * @param {string} section
     * @param {string} name
     * @param {AnyT} value
     * @returns {AnyAction}
     */
    FormActionBuilder.buildChangeSimpleAction = function (section, name, value) {
        return {
            type: this.buildChangeActionType(section),
            data: util_1.applySection(section, { name: name, value: value }),
        };
    };
    /**
     * @stable [26.08.2018]
     * @param {IKeyValue} changes
     * @returns {FieldChangeEntityT}
     */
    FormActionBuilder.buildChangesPayload = function (changes) {
        return {
            fields: Object
                .keys(changes)
                .map(function (name) { return ({ name: name, value: Reflect.get(changes, name) }); }),
        };
    };
    return FormActionBuilder;
}());
exports.FormActionBuilder = FormActionBuilder;
//# sourceMappingURL=form-action.builder.js.map