"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../../definitions.interface");
var util_1 = require("../../util");
var error_1 = require("../../error");
var list_action_builder_1 = require("./list-action.builder");
var list_interface_1 = require("./list.interface");
var api_1 = require("../../api");
function listReducer(state, action) {
    if (state === void 0) { state = list_interface_1.INITIAL_APPLICATION_LIST_STATE; }
    var _a, _b, _c;
    var section = util_1.toSection(action);
    var modifyData = action.data;
    var modifyDataPayload = modifyData && modifyData.payload;
    var updatedData;
    switch (action.type) {
        case list_action_builder_1.ListActionBuilder.buildChangeSortDirectionActionType(section):
            var sortDirectionEntityWrapper = action.data;
            return __assign({}, state, { directions: __assign({}, state.directions, (_a = {}, _a[sortDirectionEntityWrapper.payload.name] = sortDirectionEntityWrapper.payload.direction, _a)) });
        case list_action_builder_1.ListActionBuilder.buildChangeActionType(section):
            var fieldChangeEntity = action.data;
            var fieldChangeEntityId = fieldChangeEntity.rawData.id;
            return __assign({}, state, { changes: __assign({}, state.changes, (_b = {}, _b[fieldChangeEntityId] = __assign({}, (state.changes || {})[fieldChangeEntityId], (_c = {}, _c[fieldChangeEntity.name] = fieldChangeEntity.value, _c)), _b)) });
        case list_action_builder_1.ListActionBuilder.buildFirstPageActionType(section):
            return __assign({}, state, { lockPage: true, page: definitions_interface_1.FIRST_PAGE });
        case list_action_builder_1.ListActionBuilder.buildLastPageActionType(section):
            return __assign({}, state, { lockPage: true, page: Math.ceil(state.totalCount / state.pageSize) });
        case list_action_builder_1.ListActionBuilder.buildPreviousPageActionType(section):
            return __assign({}, state, { lockPage: true, page: --state.page });
        case list_action_builder_1.ListActionBuilder.buildNextPageActionType(section):
            return __assign({}, state, { lockPage: true, page: ++state.page });
        case list_action_builder_1.ListActionBuilder.buildLazyLoadActionType(section):
            return __assign({}, state, { progress: true });
        case list_action_builder_1.ListActionBuilder.buildUnTouchActionType(section):
            return __assign({}, state, { touched: false });
        case list_action_builder_1.ListActionBuilder.buildLoadActionType(section):
            return __assign({}, list_interface_1.INITIAL_APPLICATION_LIST_STATE, { progress: true, touched: true, lockPage: state.lockPage });
        case list_action_builder_1.ListActionBuilder.buildCancelLoadActionType(section):
            /**
             * @stable [31.08.2018]
             */
            return __assign({}, state, { progress: false, touched: !R.isNil(state.data) });
        case list_action_builder_1.ListActionBuilder.buildDestroyActionType(section):
            return __assign({}, list_interface_1.INITIAL_APPLICATION_LIST_STATE);
        case list_action_builder_1.ListActionBuilder.buildLoadDoneActionType(section):
            var listEntity = action.data;
            return __assign({}, state, { progress: false, lockPage: false }, (Array.isArray(listEntity)
                ? {
                    data: listEntity,
                    totalCount: listEntity.length,
                    pageSize: null,
                }
                : listEntity), { page: state.lockPage ? listEntity.page : list_interface_1.INITIAL_APPLICATION_LIST_STATE.page });
        case list_action_builder_1.ListActionBuilder.buildLazyLoadErrorActionType(section):
        case list_action_builder_1.ListActionBuilder.buildLoadErrorActionType(section):
            return __assign({}, list_interface_1.INITIAL_APPLICATION_LIST_STATE, { error: error_1.convertError(action.error).message });
        case list_action_builder_1.ListActionBuilder.buildSelectActionType(section):
            return __assign({}, state, { selected: util_1.toType(action.data).selected });
        case list_action_builder_1.ListActionBuilder.buildCreateActionType(section):
        case list_action_builder_1.ListActionBuilder.buildDeselectActionType(section):
            return __assign({}, state, { selected: null });
        case list_action_builder_1.ListActionBuilder.buildLazyLoadDoneActionType(section):
            var lazyLoadedEntity = action.data;
            modifyDataPayload = {
                id: lazyLoadedEntity.id,
                changes: lazyLoadedEntity,
            };
        // No breaks! Go to update the entity.
        case list_action_builder_1.ListActionBuilder.buildMergeActionType(section):
        // No breaks! Go to update the entity.
        case list_action_builder_1.ListActionBuilder.buildInsertActionType(section):
            if (R.isNil(modifyDataPayload.id) || R.isNil(R.find(function (item) { return item.id === modifyDataPayload.id; }, state.data || []))) {
                var insertedItem = __assign({}, modifyDataPayload.changes);
                updatedData = (state.data || []).concat(insertedItem);
                return __assign({}, state, { data: updatedData, totalCount: ++state.totalCount, selected: insertedItem });
            }
        // No breaks! Go to update the entity.
        case list_action_builder_1.ListActionBuilder.buildUpdateActionType(section):
            var mergeStrategy_1 = (R.isNil(modifyDataPayload.mergeStrategy)
                || modifyDataPayload.mergeStrategy === api_1.EntityOnSaveMergeStrategyEnum.MERGE)
                ? api_1.EntityOnSaveMergeStrategyEnum.MERGE
                : api_1.EntityOnSaveMergeStrategyEnum.OVERRIDE;
            if (state.data && state.data.length) {
                updatedData = state.data.map(function (item) { return (item.id === modifyDataPayload.id
                    ? (mergeStrategy_1 === api_1.EntityOnSaveMergeStrategyEnum.OVERRIDE
                        ? __assign({}, modifyDataPayload.changes) : __assign({}, item, modifyDataPayload.changes))
                    : item); });
                return __assign({}, state, { selected: state.selected
                        ? updatedData.find(function (item) { return item.id === state.selected.id; }) || state.selected
                        : state.selected, data: updatedData, progress: false });
            }
            else {
                return __assign({}, state, { selected: mergeStrategy_1 === api_1.EntityOnSaveMergeStrategyEnum.OVERRIDE
                        ? __assign({}, modifyDataPayload.changes) : __assign({}, state.selected, modifyDataPayload.changes), progress: false });
            }
        case list_action_builder_1.ListActionBuilder.buildRemoveActionType(section):
            var entityToRemovePayload_1 = action.data;
            var filterFn = function (entity) { return !(entity === entityToRemovePayload_1 || entity.id === entityToRemovePayload_1.removed.id); };
            var isSelectedExist = !R.isNil(state.selected);
            var filteredData = state.data.filter(filterFn);
            return __assign({}, state, { selected: isSelectedExist && filterFn(state.selected) && state.selected
                    || (isSelectedExist && filteredData.length > 0 ? filteredData[0] : null), data: filteredData, totalCount: --state.totalCount });
        default:
            return state;
    }
}
exports.listReducer = listReducer;
//# sourceMappingURL=list.reducer.js.map