"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var di_1 = require("../../di");
var util_1 = require("../../util");
var StickyHeaderPlugin = /** @class */ (function () {
    /**
     * @stable [13.12.2018]
     * @param {IUniversalScrollableComponent} component
     */
    function StickyHeaderPlugin(component) {
        var _this = this;
        this.component = component;
        component.onScroll = util_1.sequence(component.onScroll, function () { return util_1.setStickyElementProperties(_this.stickyElementInitialProperties); });
    }
    /**
     * @stable [13.12.2018]
     */
    StickyHeaderPlugin.prototype.componentDidMount = function () {
        this.stickyElementInitialProperties = this.getStickyElementInitialProperties();
        if (!R.isNil(this.stickyElementInitialProperties)) {
            this.selfElementHeight = this.selfHeight;
        }
    };
    /**
     * @stable [13.12.2018]
     */
    StickyHeaderPlugin.prototype.componentWillUnmount = function () {
        this.stickyElementInitialProperties = null;
    };
    /**
     * @stable [13.12.2018]
     * @param {Readonly<{}>} prevProps
     * @param {Readonly<{}>} prevState
     * @param {never} prevContext
     */
    StickyHeaderPlugin.prototype.componentDidUpdate = function (prevProps, prevState, prevContext) {
        this.doUpdate();
    };
    Object.defineProperty(StickyHeaderPlugin.prototype, "selfHeight", {
        /**
         * @stable [13.12.2018]
         * @returns {number}
         */
        get: function () {
            return this.domAccessor.getHeight(this.component.getSelf());
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [13.12.2018]
     * @returns {IStickyElementPayloadEntity}
     */
    StickyHeaderPlugin.prototype.getStickyElementInitialProperties = function () {
        return util_1.getStickyElementInitialProperties(this.domAccessor.toJqEl(this.component.getSelf()));
    };
    /**
     * @stable [20.12.2018]
     */
    StickyHeaderPlugin.prototype.doUpdate = function () {
        var selfHeight = this.selfHeight;
        var stickyElementInitialProperties0 = this.getStickyElementInitialProperties();
        var hasStickyElementChanged = !R.isNil(stickyElementInitialProperties0)
            && !R.isNil(this.stickyElementInitialProperties)
            && stickyElementInitialProperties0.jqStickyEl.get()[0] !== this.stickyElementInitialProperties.jqStickyEl.get()[0];
        /**
         * Only this way we can detect the DOM changes
         */
        if (selfHeight !== this.selfElementHeight || hasStickyElementChanged) {
            var stickyElementInitialProperties = this.getStickyElementInitialProperties();
            if (!R.isNil(stickyElementInitialProperties)) {
                this.selfElementHeight = selfHeight;
                this.stickyElementInitialProperties = stickyElementInitialProperties;
            }
        }
    };
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.DomAccessor),
        __metadata("design:type", Object)
    ], StickyHeaderPlugin.prototype, "domAccessor", void 0);
    return StickyHeaderPlugin;
}());
exports.StickyHeaderPlugin = StickyHeaderPlugin;
//# sourceMappingURL=sticky-header.plugin.js.map