"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var util_1 = require("../util");
var calc_1 = require("./calc");
/**
 * @stable [25.10.2018]
 * @param {AnyT} condition
 * @param {string | (() => string)} result
 * @returns {string}
 */
exports.orEmpty = function (condition, result) { return condition ? calc_1.calc(result) : ''; };
function orNull(condition, result) {
    return condition ? (util_1.isFn(result) ? result() : result) : null;
}
exports.orNull = orNull;
function orUndef(condition, result) {
    return condition ? (util_1.isFn(result) ? result() : result) : undefined;
}
exports.orUndef = orUndef;
/**
 * @deprecated
 */
function orDefault(cond, result1, result2) {
    return cond
        ? (util_1.isFn(result1) ? result1() : result1)
        : (util_1.isFn(result2) ? result2() : result2);
}
exports.orDefault = orDefault;
// TODO deprecated - use ifNotNilThanValue
/**
 * @deprecated
 */
exports.ifNilReturnDefault = function (checkedValue, result1, result2) {
    return orDefault(!R.isNil(checkedValue), result1, result2);
};
/**
 * @stable [01.12.2018]
 * @param {AnyT} checkedValue
 * @param {(() => TResult) | TResult} result
 * @returns {void | TResult}
 */
exports.ifNotNilReturnValue = function (checkedValue, result) {
    return exports.ifNilReturnDefault(checkedValue, result, null);
};
/**
 * @stable [19.08.2018]
 * @param {AnyT} value
 * @param {(() => TResult) | TResult} result
 * @returns {string | TResult}
 */
exports.ifNilReturnUuid = function (value, result) {
    return exports.ifNilReturnDefault(value, result, function () { return util_1.uuid(); });
};
/**
 * @stable [23.01.2018]
 * @param {TValue} value
 * @param {(value: TValue) => TResult} callback
 * @param {any} defaultValue
 * @returns {TResult}
 */
exports.ifNotNilThanValue = function (value, callback, defaultValue) {
    if (defaultValue === void 0) { defaultValue = null; }
    return !R.isNil(value) ? callback(value) : defaultValue;
};
//# sourceMappingURL=cond.js.map