"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../definitions.interface");
var type_1 = require("./type");
function cloneUsingFilters(source) {
    var predicates = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        predicates[_i - 1] = arguments[_i];
    }
    var sourceWithObjectValues = exports.filterByPredicate.apply(void 0, [source].concat(predicates.concat(exports.OBJECT_VALUE_PREDICATE)));
    var result = __assign({}, exports.filterByPredicate.apply(void 0, [source].concat(predicates.concat(exports.NOT_OBJECT_VALUE_PREDICATE))));
    Object.keys(sourceWithObjectValues).forEach(function (key) {
        result[key] = cloneUsingFilters.apply(void 0, [sourceWithObjectValues[key]].concat(predicates));
    });
    return result;
}
exports.cloneUsingFilters = cloneUsingFilters;
/**
 * @stable [11.08.2018]
 * @param {TSource} source
 * @param {KeyPredicateT} predicates
 * @returns {TResult}
 */
exports.filterByPredicate = function (source) {
    var predicates = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        predicates[_i - 1] = arguments[_i];
    }
    return Object.freeze(R.pickBy(function (value, key) { return predicates.length > 1
        ? predicates
            .reduce(function (previousValue, currentValue) {
            return currentValue(key, value)
                && (type_1.isFn(previousValue)
                    ? previousValue(key, value)
                    : previousValue);
        })
        : predicates[0](key, value); }, source));
};
function excludeFieldsPredicateFactory() {
    var fields = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        fields[_i] = arguments[_i];
    }
    return function (key, value) { return !fields.includes(key); };
}
exports.excludeFieldsPredicateFactory = excludeFieldsPredicateFactory;
exports.EXCLUDE_ID_FIELD_PREDICATE = excludeFieldsPredicateFactory(definitions_interface_1.ID_FIELD_NAME);
exports.EXCLUDE_EFFECTOR_FIELD_PREDICATE = excludeFieldsPredicateFactory(definitions_interface_1.EFFECTOR_FIELD_NAME);
exports.EXCLUDE_TIME_FIELDS_PREDICATE = excludeFieldsPredicateFactory(definitions_interface_1.TIME_FIELD_NAME, definitions_interface_1.FROM_TIME_FIELD_NAME, definitions_interface_1.TO_TIME_FIELD_NAME);
/**
 * @test
 * @param {TSource} source
 * @returns {TResult}
 */
function cloneUsingTimeFieldsFilters(source) {
    return cloneUsingFilters(source, exports.EXCLUDE_TIME_FIELDS_PREDICATE);
}
exports.cloneUsingTimeFieldsFilters = cloneUsingTimeFieldsFilters;
exports.OBJECT_VALUE_PREDICATE = function (key, value) { return type_1.isObject(value); };
exports.NOT_OBJECT_VALUE_PREDICATE = function (key, value) { return !type_1.isObject(value); };
exports.NOT_PROGRESS_FIELD_PREDICATE = function (key, value) { return key !== definitions_interface_1.PROGRESS_FIELD_NAME; };
exports.NOT_PASSWORD_FIELD_PREDICATE = function (key, value) { return key !== definitions_interface_1.PASSWORD_FIELD_NAME; };
/**
 * @stable [09.01.2019]
 * @param {IEntity} entity1
 * @param {IEntity} entity2
 * @returns {boolean}
 * @constructor
 */
exports.SAME_ENTITY_PREDICATE = function (entity1, entity2) { return entity1 === entity2 || entity1.id === entity2.id; };
/**
 * @stable [03.10.2018]
 * @param {string} key
 * @param {AnyT} value
 * @returns {boolean}
 * @constructor
 */
exports.DEF_KEY_VALUE_PREDICATE = function (key, value) { return type_1.isDef(value); };
/**
 * @stable [03.10.2018]
 * @param {AnyT} value
 * @returns {boolean}
 * @constructor
 */
exports.DEF_VALUE_PREDICATE = function (value) { return type_1.isDef(value); };
/**
 * @stable [27.10.2018]
 * @param {AnyT} value
 * @returns {boolean}
 * @constructor
 */
exports.NOT_NIL_VALUE_PREDICATE = function (value) { return !R.isNil(value); };
/**
 * @stable [19.12.2018]
 * @param {AnyT} value
 * @returns {boolean}
 * @constructor
 */
exports.TRUE_VALUE_PREDICATE = function (value) { return !!value; };
/**
 * @stable [03.10.2018]
 * @param {TValue[]} data
 * @param {ValuePredicateT} predicates
 * @returns {TValue[]}
 */
exports.filterArray = function (data) {
    var predicates = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        predicates[_i - 1] = arguments[_i];
    }
    return R.filter(function (value) { return predicates.length > 1
        ? predicates
            .reduce(function (previousValue, currentValue) { return currentValue(value)
            && (type_1.isFn(previousValue) ? previousValue(value) : previousValue); })
        : predicates[0](value); }, data);
};
/**
 * @stable [10.11.2018]
 * @param {TValue} data
 * @returns {TValue[]}
 */
exports.defValuesArrayFilter = function () {
    var data = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        data[_i] = arguments[_i];
    }
    return exports.filterArray(data, exports.DEF_VALUE_PREDICATE);
};
/**
 * @stable [29.11.2018]
 * @param {TValue} data
 * @returns {TValue[]}
 */
exports.notNilValuesArrayFilter = function () {
    var data = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        data[_i] = arguments[_i];
    }
    return exports.filterArray(data, exports.NOT_NIL_VALUE_PREDICATE);
};
/**
 * @stable [19.12.2018]
 * @param {TValue} data
 * @returns {TValue[]}
 */
exports.trueValuesArrayFilter = function () {
    var data = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        data[_i] = arguments[_i];
    }
    return exports.filterArray(data, exports.TRUE_VALUE_PREDICATE);
};
function excludeFieldsFilter(source) {
    var fields = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        fields[_i - 1] = arguments[_i];
    }
    return exports.filterByPredicate(source, excludeFieldsPredicateFactory.apply(void 0, fields));
}
exports.excludeFieldsFilter = excludeFieldsFilter;
exports.defValuesFilter = function (source) {
    return exports.filterByPredicate(source, exports.DEF_KEY_VALUE_PREDICATE);
};
exports.excludeIdFieldFilter = function (source) {
    return exports.filterByPredicate(source, exports.EXCLUDE_ID_FIELD_PREDICATE);
};
exports.excludeEffectorFieldFilter = function (source) {
    return exports.filterByPredicate(source, exports.EXCLUDE_EFFECTOR_FIELD_PREDICATE);
};
/**
 * @stable [16.06.2018]
 * @param {TEntity[] | TEntity} data
 * @param {IFilterAndSorterConfiguration} config
 * @returns {TEntity[]}
 */
exports.filterAndSortEntities = function (data, config) {
    if (R.isNil(data)) {
        return data;
    }
    var entities = [].concat(data);
    if (!R.isNil(config)) {
        if (!R.isNil(config.filter)) {
            entities = R.filter(config.filter, entities);
        }
        if (!R.isNil(config.sorter)) {
            entities = R.sort(config.sorter, entities);
        }
    }
    return entities;
};
/**
 * @stable [08.06.2018]
 * @param {string} query
 * @param {EntityIdT} items
 * @returns {boolean}
 */
exports.queryFilter = function (query) {
    var items = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        items[_i - 1] = arguments[_i];
    }
    if (R.isNil(query) || R.isEmpty(query)) {
        return true;
    }
    var result = false;
    items.map(function (v) { return (type_1.isString(v) ? v : R.isNil(v) ? '' : String(v)).toUpperCase(); })
        .forEach(function (v) { return result = result || v.includes(query.toUpperCase()); });
    return result;
};
//# sourceMappingURL=filter.js.map